/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global document, tizen, console, window*/

/**
 * Main application module.
 * Provides a namespace for other application modules.
 * Handles application life cycle.
 *
 * @module app
 * @requires {@link app.ui}
 * @namespace app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineApp(app) {
    'use strict';

    /**
     * Closes the application.
     *
     * @memberof app
     * @private
     */
    function exit() {
        try {
            tizen.application.getCurrentApplication().exit();
        } catch (error) {
            console.warn('Application exit failed. ', error.message);
        }
    }

    /**
     * Handles user interaction through a special point of hardware.
     *
     * If back action is detected, control is passed to the user interface
     * module first. If it decides to not handle it, the application is closed.
     *
     * @memberof app
     * @private
     * @param {CustomEvent} e
     */
    function onHwKeyEvent(e) {
        if (e.keyName === 'back') {
            if (!app.ui.handleBackAction()) {
                exit();
            }
        }
    }

    /**
     * Registers event listeners.
     *
     * @memberof app
     * @private
     */
    function bindEvents() {
        document.addEventListener('tizenhwkey', onHwKeyEvent);
    }

    /**
     * Initializes the module.
     *
     * @memberof app
     * @private
     */
    function init() {
        app.ui.init();
        bindEvents();
        app.ui.run();
    }

    /**
     * Runs the application.
     *
     * @memberof app
     * @public
     */
    app.run = function run() {
        init();
    };

    window.addEventListener('load', app.run);

})(window.app);
